Package: CohortMethod
Type: Package
Title: Comparative Cohort Method with Large Scale Propensity and
        Outcome Models
Version: 6.0.2
Date: 2026-05-07
Authors@R: c(
	person("Martijn", "Schuemie", , "schuemie@ohdsi.org", role = c("aut", "cre")),
	person("Marc", "Suchard", role = c("aut")),
	person("Patrick", "Ryan", role = c("aut"))
	)
Maintainer: Martijn Schuemie <schuemie@ohdsi.org>
Description: Functions for performing comparative cohort studies
	in an observational database in the Observational Medical Outcomes Partnership (OMOP) Common 
	Data Model. Can extract all necessary data from a database. This implements large-scale
	propensity scores (LSPS) as described in Tian et al. (2018) <doi:10.1093/ije/dyy120>,
	using a large set of covariates, including for example all drugs, diagnoses, procedures, 
	as well as age, comorbidity indexes, etc. Large scale regularized regression is used to 
	fit the propensity and outcome models as described in Suchard et al. (2013) <doi:10.1145/2414416.2414791>. 
	Functions are included for trimming, stratifying, (variable and fixed ratio) matching and 
	weighting by propensity scores, as well as diagnostic functions, such as propensity score distribution 
	plots and plots showing covariate balance before and after matching and/or trimming. Supported 
	outcome models are (conditional) logistic regression, (conditional) Poisson regression, and 
	(stratified) Cox regression. Also included are Kaplan-Meier plots that can adjust for 
	the stratification or matching.
License: Apache License 2.0
VignetteBuilder: knitr
URL: https://ohdsi.github.io/CohortMethod/,
        https://github.com/OHDSI/CohortMethod
BugReports: https://github.com/OHDSI/CohortMethod/issues
Depends: R (>= 4.1.0), DatabaseConnector (>= 6.0.0), Cyclops (>=
        3.6.0), FeatureExtraction (>= 3.0.0), Andromeda (>= 0.6.3)
Imports: methods, utils, ggplot2, gridExtra, grid, readr, plyr, dplyr,
        rlang, Rcpp (>= 0.11.2), SqlRender (>= 1.18.0), survival,
        ParallelLogger (>= 3.4.2), checkmate, EmpiricalCalibration,
        jsonlite, R6, digest
Suggests: testthat, pROC, knitr, rmarkdown, Eunomia, zip, withr,
        R.utils, RSQLite, ResultModelManager, markdown, PSweight
SystemRequirements: Java
LinkingTo: Rcpp
NeedsCompilation: yes
RoxygenNote: 7.3.3
Encoding: UTF-8
Packaged: 2026-05-08 09:50:16 UTC; schuemie
Author: Martijn Schuemie [aut, cre],
  Marc Suchard [aut],
  Patrick Ryan [aut]
Repository: CRAN
Date/Publication: 2026-05-08 11:32:11 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-08 16:05:53 UTC; unix
Archs: CohortMethod.so.dSYM
