# Generated by roxygen2: do not edit by hand

S3method(coef,OutcomeModel)
S3method(confint,OutcomeModel)
S3method(print,OutcomeModel)
S3method(print,summary.CohortMethodData)
export(adjustedKm)
export(checkCmInstallation)
export(computeCovariateBalance)
export(computeEquipoise)
export(computeMdrr)
export(computePsAuc)
export(convertUntypedListToCmAnalysesSpecifications)
export(createCmAnalysesSpecifications)
export(createCmAnalysis)
export(createCmDiagnosticThresholds)
export(createCmTable1)
export(createCohortMethodDataSimulationProfile)
export(createComputeCovariateBalanceArgs)
export(createCreatePsArgs)
export(createCreateStudyPopulationArgs)
export(createDefaultMultiThreadingSettings)
export(createFitOutcomeModelArgs)
export(createGetDbCohortMethodDataArgs)
export(createMatchOnPsArgs)
export(createMultiThreadingSettings)
export(createOutcome)
export(createPs)
export(createResultsDataModel)
export(createStratifyByPsArgs)
export(createStudyPopulation)
export(createTargetComparatorOutcomes)
export(createTrimByPsArgs)
export(createTruncateIptwArgs)
export(drawAttritionDiagram)
export(exportToCsv)
export(fitOutcomeModel)
export(getAttritionTable)
export(getDataMigrator)
export(getDbCohortMethodData)
export(getDefaultCmTable1Specifications)
export(getDiagnosticsSummary)
export(getFileReference)
export(getFollowUpDistribution)
export(getGeneralizabilityTable)
export(getInteractionResultsSummary)
export(getOutcomeModel)
export(getPsModel)
export(getResultsDataModelSpecifications)
export(getResultsSummary)
export(isCohortMethodData)
export(loadCmAnalysisList)
export(loadCohortMethodData)
export(loadTargetComparatorOutcomesList)
export(matchOnPs)
export(migrateDataModel)
export(plotCovariateBalanceOfTopVariables)
export(plotCovariateBalanceScatterPlot)
export(plotCovariatePrevalence)
export(plotFollowUpDistribution)
export(plotKaplanMeier)
export(plotPs)
export(plotTimeToEvent)
export(runCmAnalyses)
export(saveCmAnalysisList)
export(saveCohortMethodData)
export(saveTargetComparatorOutcomesList)
export(simulateCohortMethodData)
export(stratifyByPs)
export(trimByPs)
export(truncateIptw)
export(uploadResults)
exportClasses(CohortMethodData)
exportMethods(show)
exportMethods(summary)
import(Andromeda)
import(Cyclops)
import(DatabaseConnector)
import(FeatureExtraction)
import(R6)
import(dplyr)
importFrom(Rcpp,evalCpp)
importFrom(SqlRender,loadRenderTranslateSql)
importFrom(SqlRender,translateSql)
importFrom(grDevices,rgb)
importFrom(graphics,text)
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,density)
importFrom(stats,lm)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rexp)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,uniroot)
importFrom(survival,Surv)
importFrom(survival,survfit)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
useDynLib(CohortMethod)
