# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

matchPsInternal <- function(propensityScores, treatment, maxRatio, caliper) {
    .Call('_CohortMethod_matchPsInternal', PACKAGE = 'CohortMethod', propensityScores, treatment, maxRatio, caliper)
}

aucWithCi <- function(propensityScores, treatment) {
    .Call('_CohortMethod_aucWithCi', PACKAGE = 'CohortMethod', propensityScores, treatment)
}

aucWithoutCi <- function(propensityScores, treatment) {
    .Call('_CohortMethod_aucWithoutCi', PACKAGE = 'CohortMethod', propensityScores, treatment)
}

#' Compute a weight-adjusted Kaplan-Meier curve
#'
#' @param weight      Vector of observation weights
#' @param time        Vector of event times
#' @param y           Vector outcomes (0 indicates censoring, 1 indicates event-of-interest)
#'
#' @export
#'
adjustedKm <- function(weight, time, y) {
    .Call('_CohortMethod_adjustedKm', PACKAGE = 'CohortMethod', weight, time, y)
}

