% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SettingsObjects.R
\name{createTargetComparatorOutcomes}
\alias{createTargetComparatorOutcomes}
\title{Create target-comparator-outcomes combinations.}
\usage{
createTargetComparatorOutcomes(
  targetId,
  comparatorId,
  outcomes,
  nestingCohortId = NULL,
  excludedCovariateConceptIds = c(),
  includedCovariateConceptIds = c()
)
}
\arguments{
\item{targetId}{A cohort ID identifying the target exposure in the exposure
table.}

\item{comparatorId}{A cohort ID identifying the comparator exposure in the exposure
table.}

\item{outcomes}{A list of object of type \code{Outcome} as created by
\code{\link[=createOutcome]{createOutcome()}}.}

\item{nestingCohortId}{(Optional) the nesting cohort ID. If provided, this will override
the nesting cohort ID used in \code{\link[=createGetDbCohortMethodDataArgs]{createGetDbCohortMethodDataArgs()}}.}

\item{excludedCovariateConceptIds}{A list of concept IDs that cannot be used to construct
covariates. This argument is to be used only for exclusion
concepts that are specific to the target-comparator combination.}

\item{includedCovariateConceptIds}{A list of concept IDs that must be used to construct
covariates. This argument is to be used only for inclusion
concepts that are specific to the target-comparator combination.}
}
\value{
An object of type \code{TargetComparatorOutcomes}.
}
\description{
Create target-comparator-outcomes combinations.
}
\details{
Create a set of hypotheses of interest, to be used with the \code{\link[=runCmAnalyses]{runCmAnalyses()}} function.
}
