% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Simulation.R
\name{simulateCohortMethodData}
\alias{simulateCohortMethodData}
\title{Generate simulated data}
\usage{
simulateCohortMethodData(profile, n = 10000)
}
\arguments{
\item{profile}{An object of type \code{CohortMethodDataSimulationProfile} as generated using the
\code{\link[=createCohortMethodDataSimulationProfile]{createCohortMethodDataSimulationProfile()}} function.}

\item{n}{The size of the population to be generated.}
}
\value{
An object of type \link{CohortMethodData}.
}
\description{
Creates a \link{CohortMethodData} object with simulated data.
}
\details{
This function generates simulated data that is in many ways similar to the original data on which
the simulation profile is based. The contains same outcome, comparator, and outcome concept IDs,
and the covariates and their 1st order statistics should be comparable.
}
